@@ -3,108 +3,13 @@ use std::collections::HashSet;
 use std::net::{IpAddr, Ipv4Addr, SocketAddr, SocketAddrV4, SocketAddrV6, ToSocketAddrs};
 
 use log::*;
-use serde::{Deserialize, Serialize};
 
 use crate::configuration::{PublicKeyWithTime, PublicPeer, StaticConfiguration};
 use crate::crypt_udp::{AddressedTo, AdvertisementPacket, LocalContactPacket, RouteDatabasePacket};
 use crate::event::Event;
+use crate::routedb::{RouteDBManager, RouteInfo};
 use crate::wg_dev::map_to_ipv6;
 
-#[derive(Serialize, Deserialize, Clone, Debug)]
-pub struct RouteInfo {
-    pub to: Ipv4Addr,
-    pub local_admin_port: u16,
-    pub hop_cnt: usize,
-    pub gateway: Option<Ipv4Addr>,
-}
-
-#[derive(Default, Debug)]
-pub struct PeerRouteDB {
-    pub version: usize,
-    nr_entries: usize,
-    pub route_for: HashMap<Ipv4Addr, RouteInfo>,
-}
-
-#[derive(Default, Debug)]
-pub struct RouteDBManager {
-    pub routedb: Option<PeerRouteDB>,
-    incoming_routedb: Option<PeerRouteDB>,
-    latest_routedb_version: Option<usize>,
-}
-impl RouteDBManager {
-    fn is_outdated(&self) -> bool {
-        self.latest_routedb_version.is_none()
-            || self.routedb.as_ref().map(|db| db.version) != self.latest_routedb_version
-    }
-    fn latest_version(&mut self, version: usize) {
-        self.latest_routedb_version = Some(version);
-    }
-    fn invalidate(&mut self) {
-        self.routedb = None;
-        self.incoming_routedb = None;
-        self.latest_routedb_version = None;
-    }
-    pub fn process_route_database(&mut self, req: RouteDatabasePacket) -> Vec<Event> {
-        let mut need_routes_update = false;
-        let mut events = vec![];
-        debug!(target: "routing", "RouteDatabase: {:#?}", req.known_routes);
-
-        // The database will be received in one to many udp packages.
-        //
-        if let Some(mut incoming_routedb) = self.incoming_routedb.take() {
-            // route database exist and must not be complete
-            //
-            if req.nr_entries != incoming_routedb.nr_entries {
-                if req.routedb_version == incoming_routedb.version {
-                    // All ok, got more entries for the database
-                    for ri in req.known_routes {
-                        incoming_routedb.route_for.insert(ri.to, ri);
-                    }
-
-                    // Check, if the database is complete
-                    if req.nr_entries == incoming_routedb.route_for.len() {
-                        need_routes_update = true;
-                        self.routedb = Some(incoming_routedb);
-                    } else {
-                        // put back the route_db into the incoming_routedb
-                        self.incoming_routedb = Some(incoming_routedb);
-                    }
-                } else {
-                    // Received packet with wrong version info
-                    warn!(target: "routing", "Mismatch of route db version, so partial db is dropped");
-                }
-            } else {
-                warn!(target: "routing", "Another packet for complete database received, so strange db is dropped");
-            }
-        } else {
-            // First packet of a new route_db. So just store it
-            let routes = req
-                .known_routes
-                .iter()
-                .map(|e| (e.to, e.clone()))
-                .collect::<HashMap<Ipv4Addr, RouteInfo>>();
-            let incoming_routedb = PeerRouteDB {
-                version: req.routedb_version,
-                nr_entries: req.nr_entries,
-                route_for: routes,
-            };
-
-            // Perhaps the database is already complete ?
-            if req.nr_entries == incoming_routedb.route_for.len() {
-                need_routes_update = true;
-                self.routedb = Some(incoming_routedb);
-            } else {
-                // put back the route_db into the incoming_routedb
-                self.incoming_routedb = Some(incoming_routedb);
-            }
-        }
-        if need_routes_update {
-            events.push(Event::UpdateRoutes);
-        }
-        events
-    }
-}
-
 pub trait Node {
     fn routedb_manager(&self) -> Option<&RouteDBManager> {
         None
@@ -239,6 +144,9 @@ impl Node for StaticPeer {
         if self.is_alive && now - self.lastseen > 240 {
             // seems to be dead
             self.is_alive = false;
+            info!(target: &self.static_peer.wg_ip.to_string(),"static peer is not alive");
+            events.push(Event::WireguardPortHop);
+            events.push(Event::UpdateWireguardConfiguration);
         }
 
         if self.is_alive {
@@ -391,23 +299,23 @@ pub enum ConnectionType {
     Dynamic {
         endpoint: Option<SocketAddr>,
     },
-    Unknown,
+    Passive,
 }
 impl ConnectionType {
     fn endpoint(&self) -> Option<SocketAddr> {
         match self {
+            ConnectionType::Passive => None,
             ConnectionType::Static { endpoint, .. } => Some(*endpoint),
             ConnectionType::Local { endpoint, .. } => Some(*endpoint),
             ConnectionType::Dynamic { endpoint } => *endpoint,
-            ConnectionType::Unknown => None,
         }
     }
     fn as_str(&self) -> &'static str {
         match self {
+            ConnectionType::Passive => "passive",
             ConnectionType::Static { .. } => "static",
             ConnectionType::Local { .. } => "local",
             ConnectionType::Dynamic { .. } => "dynamic",
-            ConnectionType::Unknown => "unknown",
         }
     }
 }
@@ -434,7 +342,7 @@ impl DynamicPeer {
         static_config: &StaticConfiguration,
         advertisement: AdvertisementPacket,
         src_addr: SocketAddr,
-    ) -> Self {
+    ) -> Option<Self> {
         let connection: ConnectionType;
         let mut local_reachable_admin_endpoint = None;
         let mut local_reachable_wg_endpoint = None;
@@ -444,8 +352,13 @@ impl DynamicPeer {
         match &advertisement.addressed_to {
             StaticAddress => {
                 // seems the peer thinks, I am a static node.
-                warn!("needs more work");
-                connection = ConnectionType::Unknown;
+                if static_config.is_static {
+                    info!("StaticAddress: needs more work");
+                    connection = ConnectionType::Passive;
+                } else {
+                    warn!("StaticAddress: needs more work");
+                    return None;
+                }
             }
             ReplyFromStaticAddress => {
                 // The peer is a static node. So the defined endpoint can be used.
@@ -456,7 +369,8 @@ impl DynamicPeer {
                         admin_endpoint: src_addr,
                     };
                 } else {
-                    connection = ConnectionType::Unknown;
+                    warn!("ReplyFromStaticAddress: reply from static address, but peer is not a static peer");
+                    return None;
                 }
 
                 // But the static peer gives us info about our visible address.
@@ -497,13 +411,13 @@ impl DynamicPeer {
                 }
             }
             WireguardAddress | ReplyFromWireguardAddress => {
-                warn!(target: &advertisement.wg_ip.to_string(), "unexpected advertisement to wireguard address for new dynamic peer");
-                connection = ConnectionType::Unknown
+                warn!(target: &advertisement.wg_ip.to_string(), "unexpected advertisement to wireguard address for new dynamic peer => ignore");
+                return None;
             }
         }
         let mut routedb_manager = RouteDBManager::default();
         routedb_manager.latest_version(advertisement.routedb_version);
-        DynamicPeer {
+        Some(DynamicPeer {
             wg_ip: advertisement.wg_ip,
             local_admin_port: advertisement.local_admin_port,
             local_wg_port: advertisement.local_wg_port,
@@ -517,7 +431,7 @@ impl DynamicPeer {
             admin_port: src_addr.port(),
             lastseen: now,
             routedb_manager,
-        }
+        })
     }
 }
 impl Node for DynamicPeer {
@@ -551,6 +465,9 @@ impl Node for DynamicPeer {
             debug!(target: "configuration", "peer {} uses {} endpoint {}", self.wg_ip, self.connection.as_str(), endpoint);
             debug!(target: &self.wg_ip.to_string(), "use {} endpoint {}", self.connection.as_str(), endpoint);
             lines.push(format!("EndPoint = {}", endpoint));
+        } else {
+            debug!(target: "configuration", "dynamic peer {} without endpoint", self.wg_ip);
+            debug!(target: &self.wg_ip.to_string(), "is dynamic peer without endpoint");
         }
         Some(lines)
     }
@@ -615,9 +532,13 @@ impl Node for DynamicPeer {
                 // and replace myself together with route update
                 events.push(Event::UpdateRoutes);
 
-                let dp =
-                    DynamicPeer::from_advertisement(now, static_config, advertisement, src_addr);
-                return (Some(Box::new(dp)), events);
+                if let Some(dp) =
+                    DynamicPeer::from_advertisement(now, static_config, advertisement, src_addr)
+                {
+                    return (Some(Box::new(dp)), events);
+                } else {
+                    return (None, vec![]);
+                }
             } else {
                 warn!(target: "advertisement", "Received advertisement with old public key => Reject");
             }
@@ -636,7 +557,7 @@ impl Node for DynamicPeer {
                     warn!(target: "advertisement", "has not been sent via tunnel");
                     if advertisement.your_visible_wg_endpoint.is_some() {
                         events.push(Event::UpdateWireguardConfiguration);
-                        self.dp_visible_wg_endpoint = advertisement.your_visible_wg_endpoint;
+                        self.dp_visible_wg_endpoint = advertisement.my_visible_wg_endpoint;
                     }
                     events.push(Event::SendAdvertisement {
                         addressed_to: advertisement.addressed_to.reply(),
@@ -650,7 +571,7 @@ impl Node for DynamicPeer {
                         && advertisement.your_visible_wg_endpoint.is_some()
                     {
                         events.push(Event::UpdateWireguardConfiguration);
-                        self.dp_visible_wg_endpoint = advertisement.your_visible_wg_endpoint;
+                        self.dp_visible_wg_endpoint = advertisement.my_visible_wg_endpoint;
                     }
                 }
                 LocalAddress => {
@@ -673,13 +594,7 @@ impl Node for DynamicPeer {
                 | ReplyFromWireguardV6Address => {
                     // tunnel is ok. So check for visible wg endpoints
                     if self.dp_visible_wg_endpoint.is_none() {
-                        if let Some(endpoint) = advertisement.your_visible_wg_endpoint {
-                            events.push(Event::UpdateWireguardConfiguration);
-                            self.dp_visible_wg_endpoint = Some(endpoint);
-                        } else {
-                            warn!(target: "advertisement", "need more work");
-                            events.push(Event::ReadWireguardConfiguration);
-                        }
+                        events.push(Event::ReadWireguardConfiguration);
                     }
                 }
             }
@@ -732,6 +647,7 @@ impl DistantNode {
 impl Node for DistantNode {
     fn process_local_contact(&mut self, local: LocalContactPacket) {
         debug!(target: &self.wg_ip.to_string(), "Received local contact packet");
+        self.send_count = 0;
         self.local_ip_list = Some(local.local_ip_list);
         self.local_admin_port = Some(local.local_admin_port);
         self.visible_endpoint = local.my_visible_wg_endpoint;
@@ -744,6 +660,7 @@ impl Node for DistantNode {
             lines.push(format!("PublicKey = {}", &public_key.key));
             lines.push(format!("AllowedIPs = {}/128", map_to_ipv6(&self.wg_ip)));
             if let Some(endpoint) = self.visible_endpoint.as_ref() {
+                warn!("peer sends eventually local address as visible endpoint");
                 debug!(target: "configuration", "node {} uses visible (NAT) endpoint {}", self.wg_ip, endpoint);
                 debug!(target: &self.wg_ip.to_string(), "use visible (NAT) endpoint {}", endpoint);
                 lines.push(format!("EndPoint = {}", endpoint));
@@ -834,23 +751,28 @@ impl Node for DistantNode {
     ) -> (Option<Box<dyn Node>>, Vec<Event>) {
         let mut events = vec![];
 
-        let dp = DynamicPeer::from_advertisement(now, static_config, advertisement, src_addr);
+        let reply = advertisement.addressed_to.reply();
+        if let Some(dp) =
+            DynamicPeer::from_advertisement(now, static_config, advertisement, src_addr)
+        {
+            // As this peer is new, send an advertisement
+            info!(target: "advertisement", "Advertisement from new peer at old address: {}", src_addr);
+            events.push(Event::SendAdvertisement {
+                addressed_to: reply,
+                to: src_addr,
+                wg_ip: dp.wg_ip,
+            });
+            events.push(Event::UpdateWireguardConfiguration);
 
-        // As this peer is new, send an advertisement
-        info!(target: "advertisement", "Advertisement from new peer at old address: {}", src_addr);
-        events.push(Event::SendAdvertisement {
-            addressed_to: AddressedTo::WireguardAddress,
-            to: src_addr,
-            wg_ip: dp.wg_ip,
-        });
-        events.push(Event::UpdateWireguardConfiguration);
+            // if still not known, then ask wireguard
+            if dp.dp_visible_wg_endpoint.is_none() {
+                events.push(Event::ReadWireguardConfiguration);
+            }
 
-        // if still not known, then ask wireguard
-        if dp.dp_visible_wg_endpoint.is_none() {
-            events.push(Event::ReadWireguardConfiguration);
+            (Some(Box::new(dp)), events)
+        } else {
+            (None, events)
         }
-
-        (Some(Box::new(dp)), events)
     }
     fn update_from_wireguard_configuration(
         &mut self,
@@ -858,7 +780,17 @@ impl Node for DistantNode {
     ) {
         if let Some(public_key) = self.public_key.as_ref() {
             if let Some(endpoint) = pubkey_to_endpoint.remove(&public_key.key) {
-                self.visible_endpoint = Some(endpoint);
+                let mut is_local = false;
+
+                for ip in self.local_ip_list.as_ref().unwrap().iter() {
+                    if *ip == endpoint.ip() {
+                        is_local = true;
+                        break;
+                    }
+                }
+                if !is_local {
+                    self.visible_endpoint = Some(endpoint);
+                }
             }
         }
     }
